# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::Compute::Mgmt::V2020_12_01
  #
  # Compute Client
  #
  class VirtualMachineImagesEdgeZone
    include MsRestAzure

    #
    # Creates and initializes a new instance of the VirtualMachineImagesEdgeZone class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [ComputeManagementClient] reference to the ComputeManagementClient
    attr_reader :client

    #
    # Gets a virtual machine image in an edge zone.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param skus [String] A valid image SKU.
    # @param version [String] A valid image SKU version.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [VirtualMachineImage] operation results.
    #
    def get(location, edge_zone, publisher_name, offer, skus, version, custom_headers:nil)
      response = get_async(location, edge_zone, publisher_name, offer, skus, version, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a virtual machine image in an edge zone.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param skus [String] A valid image SKU.
    # @param version [String] A valid image SKU version.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(location, edge_zone, publisher_name, offer, skus, version, custom_headers:nil)
      get_async(location, edge_zone, publisher_name, offer, skus, version, custom_headers:custom_headers).value!
    end

    #
    # Gets a virtual machine image in an edge zone.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param skus [String] A valid image SKU.
    # @param version [String] A valid image SKU version.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(location, edge_zone, publisher_name, offer, skus, version, custom_headers:nil)
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'edge_zone is nil' if edge_zone.nil?
      fail ArgumentError, 'publisher_name is nil' if publisher_name.nil?
      fail ArgumentError, 'offer is nil' if offer.nil?
      fail ArgumentError, 'skus is nil' if skus.nil?
      fail ArgumentError, 'version is nil' if version.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/edgeZones/{edgeZone}/publishers/{publisherName}/artifacttypes/vmimage/offers/{offer}/skus/{skus}/versions/{version}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'location' => location,'edgeZone' => edge_zone,'publisherName' => publisher_name,'offer' => offer,'skus' => skus,'version' => version,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Compute::Mgmt::V2020_12_01::Models::VirtualMachineImage.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a list of all virtual machine image versions for the specified location,
    # edge zone, publisher, offer, and SKU.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param skus [String] A valid image SKU.
    # @param expand [String] The expand expression to apply on the operation.
    # @param top [Integer] An integer value specifying the number of images to
    # return that matches supplied values.
    # @param orderby [String] Specifies the order of the results returned.
    # Formatted as an OData query.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array] operation results.
    #
    def list(location, edge_zone, publisher_name, offer, skus, expand:nil, top:nil, orderby:nil, custom_headers:nil)
      response = list_async(location, edge_zone, publisher_name, offer, skus, expand:expand, top:top, orderby:orderby, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a list of all virtual machine image versions for the specified location,
    # edge zone, publisher, offer, and SKU.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param skus [String] A valid image SKU.
    # @param expand [String] The expand expression to apply on the operation.
    # @param top [Integer] An integer value specifying the number of images to
    # return that matches supplied values.
    # @param orderby [String] Specifies the order of the results returned.
    # Formatted as an OData query.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(location, edge_zone, publisher_name, offer, skus, expand:nil, top:nil, orderby:nil, custom_headers:nil)
      list_async(location, edge_zone, publisher_name, offer, skus, expand:expand, top:top, orderby:orderby, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of all virtual machine image versions for the specified location,
    # edge zone, publisher, offer, and SKU.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param skus [String] A valid image SKU.
    # @param expand [String] The expand expression to apply on the operation.
    # @param top [Integer] An integer value specifying the number of images to
    # return that matches supplied values.
    # @param orderby [String] Specifies the order of the results returned.
    # Formatted as an OData query.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(location, edge_zone, publisher_name, offer, skus, expand:nil, top:nil, orderby:nil, custom_headers:nil)
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'edge_zone is nil' if edge_zone.nil?
      fail ArgumentError, 'publisher_name is nil' if publisher_name.nil?
      fail ArgumentError, 'offer is nil' if offer.nil?
      fail ArgumentError, 'skus is nil' if skus.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/edgeZones/{edgeZone}/publishers/{publisherName}/artifacttypes/vmimage/offers/{offer}/skus/{skus}/versions'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'location' => location,'edgeZone' => edge_zone,'publisherName' => publisher_name,'offer' => offer,'skus' => skus,'subscriptionId' => @client.subscription_id},
          query_params: {'$expand' => expand,'$top' => top,'$orderby' => orderby,'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = {
              client_side_validation: true,
              required: false,
              serialized_name: 'parsed_response',
              type: {
                name: 'Sequence',
                element: {
                    client_side_validation: true,
                    required: false,
                    serialized_name: 'VirtualMachineImageResourceElementType',
                    type: {
                      name: 'Composite',
                      class_name: 'VirtualMachineImageResource'
                    }
                }
              }
            }
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a list of virtual machine image offers for the specified location, edge
    # zone and publisher.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array] operation results.
    #
    def list_offers(location, edge_zone, publisher_name, custom_headers:nil)
      response = list_offers_async(location, edge_zone, publisher_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a list of virtual machine image offers for the specified location, edge
    # zone and publisher.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_offers_with_http_info(location, edge_zone, publisher_name, custom_headers:nil)
      list_offers_async(location, edge_zone, publisher_name, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of virtual machine image offers for the specified location, edge
    # zone and publisher.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_offers_async(location, edge_zone, publisher_name, custom_headers:nil)
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'edge_zone is nil' if edge_zone.nil?
      fail ArgumentError, 'publisher_name is nil' if publisher_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/edgeZones/{edgeZone}/publishers/{publisherName}/artifacttypes/vmimage/offers'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'location' => location,'edgeZone' => edge_zone,'publisherName' => publisher_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = {
              client_side_validation: true,
              required: false,
              serialized_name: 'parsed_response',
              type: {
                name: 'Sequence',
                element: {
                    client_side_validation: true,
                    required: false,
                    serialized_name: 'VirtualMachineImageResourceElementType',
                    type: {
                      name: 'Composite',
                      class_name: 'VirtualMachineImageResource'
                    }
                }
              }
            }
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a list of virtual machine image publishers for the specified Azure
    # location and edge zone.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array] operation results.
    #
    def list_publishers(location, edge_zone, custom_headers:nil)
      response = list_publishers_async(location, edge_zone, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a list of virtual machine image publishers for the specified Azure
    # location and edge zone.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_publishers_with_http_info(location, edge_zone, custom_headers:nil)
      list_publishers_async(location, edge_zone, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of virtual machine image publishers for the specified Azure
    # location and edge zone.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_publishers_async(location, edge_zone, custom_headers:nil)
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'edge_zone is nil' if edge_zone.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/edgeZones/{edgeZone}/publishers'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'location' => location,'edgeZone' => edge_zone,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = {
              client_side_validation: true,
              required: false,
              serialized_name: 'parsed_response',
              type: {
                name: 'Sequence',
                element: {
                    client_side_validation: true,
                    required: false,
                    serialized_name: 'VirtualMachineImageResourceElementType',
                    type: {
                      name: 'Composite',
                      class_name: 'VirtualMachineImageResource'
                    }
                }
              }
            }
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a list of virtual machine image SKUs for the specified location, edge
    # zone, publisher, and offer.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array] operation results.
    #
    def list_skus(location, edge_zone, publisher_name, offer, custom_headers:nil)
      response = list_skus_async(location, edge_zone, publisher_name, offer, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a list of virtual machine image SKUs for the specified location, edge
    # zone, publisher, and offer.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_skus_with_http_info(location, edge_zone, publisher_name, offer, custom_headers:nil)
      list_skus_async(location, edge_zone, publisher_name, offer, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of virtual machine image SKUs for the specified location, edge
    # zone, publisher, and offer.
    #
    # @param location [String] The name of a supported Azure region.
    # @param edge_zone [String] The name of the edge zone.
    # @param publisher_name [String] A valid image publisher.
    # @param offer [String] A valid image publisher offer.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_skus_async(location, edge_zone, publisher_name, offer, custom_headers:nil)
      fail ArgumentError, 'location is nil' if location.nil?
      fail ArgumentError, 'edge_zone is nil' if edge_zone.nil?
      fail ArgumentError, 'publisher_name is nil' if publisher_name.nil?
      fail ArgumentError, 'offer is nil' if offer.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/edgeZones/{edgeZone}/publishers/{publisherName}/artifacttypes/vmimage/offers/{offer}/skus'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'location' => location,'edgeZone' => edge_zone,'publisherName' => publisher_name,'offer' => offer,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = {
              client_side_validation: true,
              required: false,
              serialized_name: 'parsed_response',
              type: {
                name: 'Sequence',
                element: {
                    client_side_validation: true,
                    required: false,
                    serialized_name: 'VirtualMachineImageResourceElementType',
                    type: {
                      name: 'Composite',
                      class_name: 'VirtualMachineImageResource'
                    }
                }
              }
            }
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

  end
end
